
/* This program is to obtain Rohschild-Stiglitz equilibria as well as 
the critical ganma at which separating does not exist.

This program checks for both, separating and pooling deviations  */    


new;
library optmum;
#include anneal.src;

#include optmum.ext;
#include hessian.src;


output file = 2rslnnr.out reset;
@ ------------ User Parameters ------------------@
satol=1e-8;
inc=0.01;

r=1.8;
pb=0.8;
pg=0.2;
rho=0.5;

/* Uncomment this For U=ln(t) */

fn util(t)=ln(t);  @Utility function@

fn invu(t)=exp(t); @Inverse of the utility function@

fn deru(t)=1/t;    @Derivative of the utility function@

fn derinvut(t)=exp(t); @Derivative of the inverse of the utility@                 


/* For SQRT 

fn util(t)=sqrt(t);  @Utility function@

fn invu(t)=t^2; @Inverse of the utility function@

fn deru(t)=1/(2*sqrt(t));    @Derivative of the utility function@

fn derinvut(t)=2*t; @Derivative of the inverse of the utility@ */

/* For 2x-x^2 

fn util(t)=2*t-t^2; @utility function@

fn invu(t)=(2+sqrt(4-4*t))/2; @Inverse of utility. We take the positive branch@

fn deru(t)=2-2*t; @Derivative of utility@

fn derinvut(t)=-1/sqrt(4-4*t); @derivative of inverse of utility@

*/

@------------- User, do not touch ---------------@

"---------------------------------------------------------------";
"---------------------------------------------------------------";
"This Program gives the Rothschild-Stiglitz Results";

"------------------- Parameter Values --------------------------";
temp1={ r,pb,pg,rho };
temp2=r|pb|pg|rho;
call presen2(temp1,temp2);

/**********************************************************************
OBTAINNING ROTHSCHILD-STIGLITZ SEPARATING CONTRACT
************************************************************************/
B=r/(1+pb);  @This is the efficient contract for the bad type @


@                           OPTIMIZATION                                 @
_opgtol=1e-8;
_opalgr=2;
__output=0;
initheta=0.8;
"Tolerance ";;_opgtol;
flag=0;
 {theta,f,g,retcode}=optprt(optmum(&equil,initheta));

@
{theta,f,g,retcode}=qnewton(&equil,initheta);
@
/************** Checking the result of the minimization **************/

if retcode ne 0;
   "***********************************************************************
   Retcode is different from zero. It is";;retcode;
   goto final;
   endif;

Hessiano=hessian(&equil,theta);
call hessana(hessiano,-1,0);
"";
"";
glon1g=theta;
glom1g=r-pg*glon1g;
temp1={ m1g, n1g, B };
temp2=glom1g|glon1g|B;
"------ The results of the separating candidate in Rothschild Stiglitz----------";
call presen2(temp1,temp2);
"";
flag=1;
{ temp3 }=equil(theta);

"The slack in the equation is the following. It should be close to zero ";;temp3;

"";
"Press a key";;waitc;


/********* Computing the critical pooling ganma ********************/
print "-----Computing critical ganma for pooling -------";
initheta=0.6|0.5;
flag=0;
/*
{theta,f,g,retcode}=optprt(optmum(&critical,initheta));
*/
{theta,f,g,retcode}=qnewton(&critical,initheta);
/************** Checking the result of the minimization **************/

if retcode ne 0;
   "***********************************************************************
   Retcode is different from zero. It is";;retcode;
   goto final;
   endif;

Hessiano=hessian(&critical,theta);
call hessana(hessiano,-1,0);
flag=1;
{ temp3 }=critical(theta);
"The slacks of the system of equation is the following";
"It should be close to zero ";;
"Slacks: ";temp3;

"";
"";
"The Critical Ganma for Pooling in Rothschild Stiglitz is ";;theta[2];
cripool=theta[2];

"***********************************************************************";
"Now we will compute the critical ganma that makes that the candidate 
survives to a separating deviation.";
"We assume ICCB binding and Bad contract is efficient ";

U1G=exputil(glom1g,glon1g,pg);
U1B=exputil(B,B,pb);
"";
"Utility of the Bad in the candidate ";;U1B;
"Utility of the Good in the candidate ";;U1G;

ganma=cripool+inc;
flag=0;
do until flag==3;
ganma=ganma-inc;

ub={ 2, 2 };
lb={ 0, 0 };

{theta, f, nacc, nfcnev, nobds, ier, temper, vm} =
     sa (&bestres1,initheta,1,0.85,satol,20,100,4,1e+8,lb,ub,2|2,0,5,3|3);
"------------------------------------------------------------------------------";
"For ganma ";;ganma;
"The best response is ";
brmg=theta[1];
brng=theta[2];
brmb=rest2(brmg,brng,pb);
brnb=brmb;
temp1={ m0g, n0g, B };
temp2=brmg|brng|brnb;
call presen2(temp1,temp2);
"Profits ";;f;
"";
UBb1br=exputil(brmb,brnb,pb);
UBg1br=exputil(brmg,brng,pb);
UGg1br=exputil(brmg,brng,pg);
UGb1br=exputil(brmb,brnb,pg);
"";
temp1= { UBb1br, UBg1br, UGg1br, UGb1br };
temp2=Ubb1br|ubg1br|ugg1br|ugb1br;
"Utilities in the best response are ";;
call presen2(temp1,temp2);
"";
if f gt 0;
   "CANDIDATE IS BROKEN ";
elseif f le 0;
    "CANDIDATE SURVIVES ";
    flag=flag+1;
else;
    "Error 4";
     end;
endif;

endo;
crisepar=ganma;
"***********************************************************************";
"Now we will compute the critical ganma that makes that the candidate 
survives to a separating deviation.";
"We assume ICCG binding and G contract is efficient ";



ganma=cripool+inc;
flag=0;
do until flag==3 or (ganma-inc)<crisepar;
ganma=ganma-inc;

ub={ 10, 10 };
lb={ 0, 0 };

{theta, f, nacc, nfcnev, nobds, ier, temper, vm} =
     sa (&bestres2,initheta,1,0.85,satol,20,100,4,1e+8,lb,ub,2|2,0,5,3|3);
"For ganma ";;ganma;
"The best response is ";
brmb=theta[1];
brnb=theta[2];
brmg=rest3(brmb,brnb,pg);
brng=brmg;
temp1={ m0b, n0b, G };
temp2=brmb|brnb|brng;
call presen2(temp1,temp2);
"Profits ";;f;
"";
UBb1br=exputil(brmb,brnb,pb);
UBg1br=exputil(brmg,brng,pb);
UGg1br=exputil(brmg,brng,pg);
UGb1br=exputil(brmb,brnb,pg);
"";
temp1= { UBb1br, UBg1br, UGg1br, UGb1br };
temp2=Ubb1br|ubg1br|ugg1br|ugb1br;
"Utilities in the best response are ";;
call presen2(temp1,temp2);
"";
if f gt 0;
   "CANDIDATE IS BROKEN ";
elseif f le 0;
    "CANDIDATE SURVIVES ";
    flag=flag+1;
else;
    "Error 4";
     end;
endif;

endo;

/**************************** PROCEDURES ******************************/


/* Expected Utility */
proc exputil(m,n,p);
local um,un;
um=util(m);
un=util(n);
retp(um+p*un);
endp;

/* Given pg, m0b and n0b, it computes the contract
(alpha,alpha) which leaves the ICCG binding */
proc rest3(mb,nb,pg);
local a;
a=invu((util(mb)+pg*util(nb))/(1+pg));
retp(a);
endp;


/* Given pb, m0g and n0g, it computes the contract
(alpha,alpha) which leaves the ICCB binding */

proc rest2(mg,ng,pb);
local a;
a=invu((util(mg)+pb*util(ng))/(1+pb));
retp(a);
endp;

/* this procedure computes the profit of firm 0 when firm
1 is in the candidate. It assumes ICCB binding and B efficient */

proc bestres1(theta);
local m0g,n0g,n0b,m0b,profg,profb,ugg,ugb,ubb;
m0g=theta[1];
n0g=theta[2];

{ m0b }=rest2(m0g,n0g,pb); @Bad type is efficient@
n0b=m0b;
@Notice, sem0g...is the candidate. This is given @

@The ICCB is binding for sure, due to the constraint above.
The machine could search in places where the ICCG of the Good
is not verified. In this case, the relevant demand and profit
comes from the contract for the bad @

UGG=exputil(m0g,n0g,pg);
UGB=exputil(m0b,n0b,pg);
UBB=exputil(m0b,n0b,pb);

if maxc(UGG|UGB) gt U1G; @This means that G comes to firm 0@ 
    	
	IF UGG ge UGB;    @G comes and takes its contract@
        profg=r-m0g-n0g*pg;

        ELSEIF UGG lt UGB; @G comes but takes B contract@ 
        profg=r-m0b-n0b*pg; 

        ELSE;
        "Error 1";
         End;
         endif;


elseif maxc(UGG|UGB) == U1G; @This means that G is indiferent
                             between coming and not coming.
                             So, half of the profits@

        IF UGG ge UGB;    @G comes and takes its contract@
        profg=(r-m0g-n0g*pg)/2;

        ELSEIF UGG lt UGB; @G comes but takes B contract@ 
        profg=(r-m0b-n0b*pg)/2; 

        ELSE;
        "Error 1bis";
         End;
         endif;

elseif maxc(UGG|UGB) lt U1G; @This means that G does not come to firm 0@
        profg=0;

else;
       "Error 2";
        end;
endif;


if UBB gt U1B;   @This means that B comes to firm 0@
                 @We are impossing ICCB binding, so we do not need to
                  check incentives to lie@
    profb=r-m0b-n0b*pb;

elseif UBB==U1B;  @This means that B is indiferent whether to come or not@
    profb=(r-m0b-n0b*pb)/2;

elseif UBB lt U1B; @This means that B does not come to firm 0@
   profb=0;
      
else;
      "Error 3";
      end;
endif;
  
retp(ganma*profg+(1-ganma)*profb);


endp;


/* this procedure computes the profit of firm 0 when firm
1 is in the candidate. It assumes ICCG binding and G efficient */

proc bestres2(theta);
local m0g,n0g,n0b,m0b,profg,profb,ugg,ubg,ubb;
m0b=theta[1];
n0b=theta[2];

{ m0g }=rest3(m0b,n0b,pg); @G type is efficient@
n0g=m0g;


@The ICCG is binding for sure, due to the constraint above.
The machine could search in places where the ICCB of the Good
is not verified. In this case, the relevant demand and profit
comes from the contract for the G @

UBB=exputil(m0b,n0b,pb);
UBG=exputil(m0g,n0g,pb);
UGG=exputil(m0g,n0g,pg);

if maxc(UBB|UBG) gt U1B; @This means that B comes to firm 0@ 
    	
	IF UBB ge UBG;    @B comes and takes its contract@
        profb=r-m0b-n0b*pb;

        ELSEIF UBB lt UBG; @B comes but takes G contract@ 
        profb=r-m0g-n0g*pb; 

        ELSE;
        "Error 1";
         End;
         endif;

elseif maxc(UBB|UBG) == U1B; @This means that B is indiferent
                             between coming and not coming.
                             So, half of the profits@

        IF UBB ge UBG;    @B comes and takes its contract@
        profb=(r-m0b-n0b*pb)/2;

        ELSEIF UBB lt UBG; @B comes but takes G contract@ 
        profb=(r-m0g-n0g*pb)/2; 

        ELSE;
        "Error 1bis";
         End;
         endif;

elseif maxc(UBB|UBG) lt U1B; @This means that B does not come to firm 0@
        profb=0;

else;
       "Error 2";
        end;
endif;


if UGG gt U1G;   @This means that G comes to firm 0@
                 @We are impossing ICCG binding, so we do not need to
                  check incentives to lie@
    profg=r-m0g-n0g*pg;

elseif UGG==U1G;  @This means that G is indiferent whether to come or not@
    profg=(r-m0g-n0g*pg)/2;

elseif UGG lt U1G; @This means that G does not come to firm 0@
   profg=0;
      
else;
      "Error 3";
      end;
endif;
  
retp(ganma*profg+(1-ganma)*profb);


endp;


/* Procedure that is used to solve the system of 
equations that give the Separating equilibria
                               */

proc equil(a);
local n1g,sl3;

n1g=a[1];


sl3=util(r-pg*n1g)+pb*util(n1g)-util(B)-pb*util(B); @ICC@
if flag==0;                                    @Notice, B is a global@  
retp(sl3^2);
else;
retp(sl3);
endif;
endp;


/* Thid procedure is to compute the critical ganma */

proc critical(a);
local n,gam,m,sl1,sl2;
n=a[1];
gam=a[2];
m=r-gam*pg*n-(1-gam)*pb*n;
sl1=util(m)+pg*util(n)-util(glom1g)-pg*util(glon1g);
sl2=deru(m)/(pg*deru(n))-1/(gam*pg+(1-gam)*pb);
if flag==0;
retp(sl1^2+sl2^2);
else;
retp(sl1|sl2);
endif;
endp;


proc presen2(varnames,coef);
local fmt,mask,results;
fmt={ "- *.*lg" 10 10,
      "*.*lf" 16 16 };
mask={ 0,1 };
mask=mask';
cls;

print;
print "Name        Values";
print "____________________";
results=varnames~coef;
call printfm(results,mask,fmt);
retp(0);
print;
endp;


/* ***************************************************************************

This program analyse a given Hessian to obtain diagnosis related to convergence

******************************************************************************/

proc(0)=hessana(H,a,b);
/*
H=Hessian.
a=+1, if the Hessian is the result of a maximization
  -1, if the Hessian is the result of a minimization

b=1, if you require analysis of dependencies among columns
b=0, if you only require eigenvalue analysis.        */

local myeig,condnum,r,e;

myeig=eig(H);
condnum=maxc(myeig)/minc(myeig);
@
"";
"The eigenvalues are";
"";
myeig';
@
"";
if a==-1 and minc(real(myeig)) > 0;
"Eigenvalues with right sign";
"";
@
"The closer the condition number is to 1, the better your hessian";
"Condition number is ";;condnum;
"So when inverting, the number of decimal places you lose is ";
log(condnum);
@
endif;
if a==-1 and minc(real(myeig)) < 0;
"Eigenvalues with wrong sign";
"";
endif;
if a==1 and maxc(real(myeig)) < 0;
"Eigenvalues with right sign";
@
"The closer the condition number is to 1, the better your hessian";
"Condition number of the hessian is";;condnum;
"So when inverting, the number of decimal places you lose is ";
log(condnum);
@
endif;
if a==1 and maxc(real(myeig)) > 0;
"Eigenvalues with wrong sign";
endif;

if b==1;
"";
"****************************************************************************";
"  The diagonal of the R matrix is the following. The number of elements     ";
" close to zero are the number of linear dependent colums                    ";
"****************************************************************************";

"";
{R,e}=qre(H);
diag(R);
endif;
endp;


FINAL:
output off;
end;